# This code simulates coupled integrodifference equations
# in one spatial dimension 
# (some great infromation on IDEs can be found at
# http://www.math.usu.edu/powell/wauclass/labs.html)

# The goal of this code is that an undergraduate with an understanding
# of recursion equations/ODE models in ecology could edit it to consider
# a different type of basic population biology model, or a different
# type of dispersal kernel, or change the parameters or spatial resolution.

# This code is based off a MATLAB version by Christina Cobbold and code by
# Jim Powell (malthus.m) which can be found at the link above

# I have tried to write the code so that parts that you might want to change appear
# first, however this is not entirely possible. After that I have tried to identify the
# later parts that you might want to change with comments.
#################

# SI IDE MODEL
# The dynamical system is an SI or SIS epidemic model.
# Dispersal is assumed to follow a normal distribution.

# r - geometric growth rate for susceptible hosts
# d - background mortality rate
# beta - infection transmission rate
# v - disease-induced mortality rate (virluence)

r=1;
gamma = 0;
beta = 1;
d = .01;
v = 0.1;

## Parameters related to dispersal
# Dispersal is assumed to follow a normal distribution
# cS - mean dispersal distance for susceptible hosts
# cI - mean dispersal distance for infected hosts
# DS - difussion coefficient for susceptible hosts (related to variance
#      of the normal distribution
# DI - difussion coefficent for infected hosts
cS=0;
cI=0;
DS=0.25;
DI=0.1;

## Parameters related to dependent and independent variables
# nsteps - the length of time to run the simulation for
# N - an integer (whole number) that increases the resolution of the spatial grid
# L - the spatial grid runs from -L to L
# Dt - the detection threshold. This should be a small number. The pouplation
# can be detected when the density exceeds Dt.
nsteps=20;
N = 10;
L=20;
Dt = 0.01;

## BELOW SETS UP THE SPATIAL GRID, x - I wouldn't edit this part of the code.
# Change L and N (above) if you want to alter the spatial grid

# To make the code run efficiently the number of points in the domain
# should be given by the formula below. If you want to increase the spatial
# resolution of your solution you can increase N (but it should be a whole number)
np = 2^N*4;

#detla x, width of grid pts  (Note the numerical domain is twice as large (-2L<x<2L)
# The buffer at each end of the domain is to deal with the
# fact that fft is periodic and so we want to make sure the
# reflected portion does not influence our dynamics.
# NOTE: fft assumes whatever data is at x=2L is the same as the 
# data at x=-2L, so we miss off an end point when define the domain
dx = 4*L/np;
# x is the vector of the spatial values
x = seq(0,np-1,1)*dx-2*L;
###############

# The commands below define the inital distributions (you may wish to edit these)
# This makes the initial density equal to 0.1 on the interval -1 to 1 and 0 elsewhere.

# S(t,x) - the density of susceptible hosts at time, t.
# I(t,x) - the density of infected hosts at time, t.
# Sx - the furtherest spatial extent of the susceptible population at time, t.
# Ix - the furtherest spatial extent of the infected population at time, t.
# KS - the dispersal kernel for susceptible hosts
# KI - the dispersal kernel for infected hosts

S = 0.1*(abs(0.5*x)<=.5);
S0=S;
# The initial distribution of infecteds is set to be the same as susceptibles.
I=S;
I0=I;
# Preallocation of vectors that record the spatial extent of the population at time, t
Sx = NULL;
Ix = NULL;

# Define the Gaussian (or normal) dispersal kernel (you may wish to edit these)
KS = exp(-(x-cS)^2/(4*DS))/(4*pi*DS)^.5;
KI = exp(-(x-cI)^2/(4*DI))/(4*pi*DI)^.5;
# normalize KS and KI to make it have integral one
KS=KS/(dx*sum(KS));
KI=KI/(dx*sum(KI));

## FUNCTIONS
# Here I program the fftshift function and its inverse. The fftshift
# switchs the left and right halves of the spatial domain so that there
# are a lot of zeros in the center which makes the fft run more efficiently.
# fft is the fast fourier transform and it is a computationally efficient
# method for solving the integrodifference equations (IDEs).
fftshift1=function(x){
  x1 = ceiling(length(x)/2);
  y = c(x[(x1+1):length(x)],x[1:x1]);
  return(y)
}

ifftshift1=function(x){
  x1 = floor(length(x)/2);
  y = c(x[(x1+1):length(x)],x[1:x1]);
  return(y)
}

#       calculate the fft of KS and KI, multiplying by dx to account
#       for the spatial discretization.  The fftshift accounts for
#       using an interval of (-L,L) as opposed to (0,2*L).
fKS = fft(fftshift1(KS))*dx
fIK = fft(fftshift1(KI))*dx

# plot the inital distribution of susceptibles in dashed black
# plot the inital distribution of infecteds in dashed red
# (you will certainly want to change around the axes limits if you
# change the model)
par(mfrow=c(2,2))
plot(x,S0,typ="l", xlim = c(-L,L), ylim=c(0,5), xlab = "space", ylab="density", lty = 2, main = "Solutions for t=1 to nsteps")
lines(x, I0, col = "red", lty = 2)

#       begin iterating the model over time
for(t in 1:nsteps){
  # calculate the fft of S and I
  fS = fft(S);
  fI = fft(I);
  # perform the convolutions
  fKS2 = fS*fKS;
  fIK2 = fI*fIK;
  #       fKS2 and fIK2 now contains the fourier transform of the convolution;
  #       Unlike MATLAB, R returns and unnormalized
  #       ifft so for equivalent results, I divide by length(x).
 S=abs(fft(fKS2, inverse=TRUE)/length(x));
 I=abs(fft(fIK2, inverse=TRUE)/length(x));
 
 # The dynamics are for an SIS model for discrete time (you may wish to edit this if
 # you are studying a different dynamical system and you will potentially need to
 # back edit the transformations on the variables, the parameters, and some of the plots
 # below too)
 
 S = S+r*S- beta*S*I + gamma*I - d*S;
 I = I+beta*S*I - gamma*I - d*I - v*I;
 
 # Due to the discretization of time, the solutions may go slightly negative.
 # To avoid this, I force the solutions to never go negative by setting negative
 # results to zero.
 S[S<0]=0;
 I[I<0]=0;
 
 # Plot the solutions for every iteration of the time loop.
  lines(x,S,typ="l")
  lines(x,I,typ="l", col="red")
  # Determine the furtherst spatial extent of susceptible and infected populations
  Sx[t]=x[max(which(S>Dt))];
  Ix[t]=x[max(which(I>Dt))];
}
# Graph the solution at the end time (in a seperate panel)
plot(x,S, xlim = c(-L,L), typ = "l", xlab = "space", ylab="density",main="Solutions at t=nsteps")
lines(x,I, typ="l", col="red")

# To determine the spread rate, we perform a linear regression
# The population may take some time to settle into a constant spread rate.

# reg.start - the start time for the linear regression
reg.start = 5;
time = reg.start:nsteps
S.extent = Sx[reg.start:nsteps]
I.extent = Ix[reg.start:nsteps]
# lm() means linear model - this code below is performing a linear regression.
reg.S = lm(S.extent~time)
reg.I = lm(S.extent~time)

# Plot the spatial extent of susceptible and infected hosts as a function of time
# and show the linear regression.
plot(1:nsteps, Sx, xlab = "time", ylab = "spatial extent", main="The slope is the spread rate")
abline(lm(S.extent~time))
points(1:nsteps, Ix, col="red")
abline(lm(I.extent~time), col="red")

# The commands: coefficients(reg.S) and coefficients(reg.I) can be typed into the console
# to report the fitted coefficients for the linear regression on the spatial extent of the
# susceptible and infected poulations. summary(reg.S) and summary(reg.I) will also yield
# information on the regression.
coefficients(reg.S)
coefficients(reg.I)
